/* ------------------------------------------------------------------------- */
/* cpu.h                                                                     */
/*                                                                           */
/* Copyright (c) 2001, 2002 Magnus Damm <damm@opensource.se>                 */
/* All rights reserved.                                                      */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. Neither the name of the copyright holders nor the name of its          */
/*    contributors may be used to endorse or promote products derived        */
/*    from this software without specific prior written permission.          */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS   */
/* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, */
/* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR    */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT OWNER OR CONTRIBUTORS */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR    */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/* ------------------------------------------------------------------------- */

#ifndef __CPU_H__
#define __CPU_H__

#include "colibri_config.h"

#include "cpu_types.h"
#include "types.h"

typedef uint16_t cpu_flags_t;

extern inline cpu_flags_t cpu_flags_save(void)
{
	cpu_flags_t flags;

	__asm__("stc\tccr,%0h" : "=r" (flags));
	
	return flags; 
}

extern inline void cpu_flags_cli(void)
{
	__asm__("orc\t#0x80,ccr"
		:
		:
		: "cc");
}

extern inline void cpu_flags_sti(void)
{
	__asm__("andc\t#0x7f,ccr"
		:
		:
		: "cc");
}

extern inline cpu_flags_t cpu_flags_save_and_cli(void)
{
	cpu_flags_t flags;
#if 0
	__asm__("stc\tccr,%0h\n\t"
	        "orc\t#0x80,ccr" : "=r" (flags));
#else
	flags = cpu_flags_save();
	cpu_flags_cli();
#endif	
	return flags; 
}

extern inline void cpu_flags_restore(cpu_flags_t flags)
{
	__asm__("ldc\t%0h,ccr"
		: /* no output registers */ 
		: "r" (flags));

}

extern inline void cpu_idle(void)
{
	__asm__("sleep");
}

typedef uint8_t cpu_irq_nest_t;

extern cpu_irq_nest_t cpu_irq_nest;

void cpu_head_schedule(void);

extern inline void cpu_schedule(void)
{
	cpu_head_schedule();
}

void *cpu_head_context_create(void *stack_ptr, void *function, void *arg);

extern inline void *cpu_context_create(void *stack_ptr, 
				       void *function, 
				       void *arg)
{
	return cpu_head_context_create(stack_ptr, function, arg);

}

void *cpu_context_init(void *stack_buffer, 
		       uint16_t stack_size,
		       void *function,
		       void *arg);

int cpu_timer_init(void);
void cpu_timer_cleanup(void);

#define CPU_STACKSIZE_MIN (10*2) /* r0->r6, flags, pc, ret-addr */

#ifdef COLIBRI_CONFIG_CPU_3297
#define CPU_IRQ_NR 37

#define CPU_IRQ_START  0
#define CPU_IRQ_NMI    3
#define CPU_IRQ_IRQ0   4
#define CPU_IRQ_IRQ1   5
#define CPU_IRQ_IRQ2   6

#define CPU_IRQ_ICIA  12
#define CPU_IRQ_ICIB  13
#define CPU_IRQ_ICIC  14
#define CPU_IRQ_ICID  15
#define CPU_IRQ_OCIA  16
#define CPU_IRQ_OCIB  17
#define CPU_IRQ_FOVI  18

#define CPU_IRQ_CMI0A 19
#define CPU_IRQ_CMI0B 20
#define CPU_IRQ_OVI0  21

#define CPU_IRQ_CMI1A 22
#define CPU_IRQ_CMI1B 23
#define CPU_IRQ_OVI1  24

#define CPU_IRQ_ERI   27
#define CPU_IRQ_RXI   28
#define CPU_IRQ_TXI   29
#define CPU_IRQ_TEI   30

#define CPU_IRQ_ADI   35
#define CPU_IRQ_WOVF  36

#endif

#endif /* __CPU_H__ */
