/* ------------------------------------------------------------------------- */
/* sem_priority.c                                                            */
/*                                                                           */
/* Copyright (c) 2002 Magnus Damm <damm@opensource.se>                       */
/* All rights reserved.                                                      */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. Neither the name of the copyright holders nor the name of its          */
/*    contributors may be used to endorse or promote products derived        */
/*    from this software without specific prior written permission.          */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS   */
/* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, */
/* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR    */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT OWNER OR CONTRIBUTORS */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR    */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/* ------------------------------------------------------------------------- */

#include "sem_private.h"
#include "sched.h"

#ifdef COLIBRI_CONFIG_SEM_PRIORITY

/* insert() and unlock() should be called with interrupts disabled */

static void sem_priority_insert(sem_t *sem)
{
	sched_task_t *t = sched_task_running;
	sched_priority_t p = t->priority;
	sched_task_t **rp;

	/* figure out the correct position in the list */

	for (rp = &sem->queue_first; *rp; rp = &((*rp)->next)) {

		if ((*rp)->priority > p) { /* 0 highest, 255 lowest */
			break;
		}
	}

	t->next = *rp;
	*rp = t;
}

static void sem_priority_unlock(sem_t *sem)
{
	sched_task_t *t;
	
	t = sem->queue_first;
	sem->queue_first = t->next;

	sched_to_ready(t);
}

static void sem_priority_init(sem_t *sem)
{
	sem->queue_first = NULL;
}

const sem_queue_ops_t sem_priority_ops = {
	sem_priority_insert,
	sem_priority_unlock,
	sem_priority_init,
};

#endif
